% T1 fitting

% A new version of the T1 fitting programme for Bruker 9.4T NMR

% Richard Foster

% Created 17/10/16 Modified 17/10/16 Modified 11/01/17

% Calls the following script files: read_all_2dseq_v2, extract_Bruker_data
% Calls the following function files: T1fit_w_stats, Sat_rec (within
% T1fit_w_stats function)

close all
clear all

%% Definition of the path to the file

%path_name = uigetdir('C:\Documents and Settings\phyrjf\My Documents\Desktop\*.*','Select data folder - only up to proc no');
path_name = uigetdir('C:\Users\Richard\Desktop\Stuff for ISSF project\Test dataset for T1 code\20','Select data folder - only up to proc no');
methodfile = strcat(path_name, '\method');

if isequal(path_name,0);
    disp('User exited');
    break
else
    
    
    %% Enter filenames for output data
    % This will save the T1,M0, rsquared and stderr images into *.mat files in the format
    % filename_T1_slicex.mat and filename_M0_slicex.mat If no filename is entered, it will still save all
    % files to the same directory.
    outputfilename = input ('Enter Filename for output images: ', 's') ;
    
    %% Matrix dimensions
    global rdim cdim;
    dim=extract_Bruker_data('PVM_Matrix','yes',2,methodfile);
     % switched these two on 11/01/17 - didn't like anisotropic images. If
    % images look wrong, this may need switching.
    %rdim=dim(2);
    %cdim=dim(1);
    rdim=dim(1);
    cdim=dim(2);
    
    %% No. of Slices
    nSlices=extract_Bruker_data('PVM_SPackArrNSlices','yes',1,methodfile);
    
    %% COUNTERS
    
    % no. of TR experiments (no of TRs used):   nTR = NumT1Exps
    
    nTR = extract_Bruker_data('NumT1Exps','no',1,methodfile);
    
    %% Repetition times used
    
    TR = extract_Bruker_data('MultiRepetitionTime','yes',[1,nTR], methodfile)/1e3;
    
    %% Total no. of images in data set
    nImages = nTR*nSlices;
    
    %% Read in image data
    
    Image = read_all_2dseq_v2(path_name,nImages,nSlices);
    
    %% Sort Data
    
    for i=1:nTR
        % Takes the matrix volume from the images read in and sorts into TR -
        % i.e. puts all Slices for a given TR together. Note that this is
        % specific to the Bruker for a RAREVTR sequence as it outputs the files
        % as TR1 Slices 1-4, TR2 Slices 5-8 etc.
        vols{i} = Image(:,:,((nSlices*i)-(nSlices-1)):nSlices*i) ;
        %vols{i} = ROI(:,:,((numslices*i)-(numslices-1)):numslices*i) ;
        % Takes each TR volume and now reshapes into a single column vector, SI
        % Puts each TR column vector into new 2D matrix which is of size (x*y*z) by TR
        SI(:,i) = reshape(vols{i},numel(vols{i}),1) ;
    end
    
    
    %Gives number of elements in each column vector - choose 1 in case only one
    %TR!
    NPX = numel(SI(:,1)) ;
    
    %% Fit Each row in SI
    
    M0=ones(size(SI(:,1)));
    T1=ones(size(SI(:,1)));
    Rsq = ones(size(SI(:,1)));
    Stderr = ones(size(SI(:,1)));
    
    for i=1:NPX
        
        [ T1(i,1),M0(i,1),Rsq(i,1),M0err(i,1),T1err(i,1) ] = T1fit_w_stats(TR,SI(i,:) );
        Count = strcat(num2str(i) , '_fits completed of_ ' , num2str(NPX)) ;
        disp(Count);
        
    end
    
    
    
    %% Turns T1 & M0 columns back into image matrices
    
    T1_image3D = reshape(T1, size(vols{1})) ;
    M0_image3D = reshape(M0, size(vols{1})) ;
    Rsq_image3D = reshape(Rsq, size(vols{1}));
    M0err_image3D = reshape(M0err, size(vols{1}));
    T1err_image3D = reshape(T1err, size(vols{1}));
    
    %% Sort into slices and Save images
    
    %T1_image_slice = ones([cdim rdim]) ;
    %M0_image_slice = ones([cdim rdim]) ;
    
    for i=1:nSlices
        
        Ouputmatrix = zeros(size(vols{1},2),size(vols{1},1),5);
        Outputmatrix(:,:,1) = T1_image3D(:,:,i)';
        Outputmatrix(:,:,2) = M0_image3D(:,:,i)';
        Outputmatrix(:,:,3) = Rsq_image3D(:,:,i)';
        Outputmatrix(:,:,4) = M0err_image3D(:,:,i)';
        Outputmatrix(:,:,5) = T1err_image3D(:,:,i)';
        
        filename_T1 = strcat(path_name, outputfilename,'_slice', num2str(i), '.mat');
        save(filename_T1, 'Outputmatrix') ;
        
    end
end
